/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.rootfilter;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.clientinterface.*;


/**
 * Acts as a Data Handler to a CORBA based client.
 *
 * @author  $Author: tdb $
 * @version $Id: CorbaDataHandler.java,v 1.2 2004/08/01 10:41:06 tdb Exp $
 */
class CorbaDataHandler extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.2 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---
    
    /**
     * Construct a new CorbaDataHandler.
     *
     * @param client A reference to the "servant" part of the connecting client.
     * @param cchServer A reference to the class that is control us.
     */
    public CorbaDataHandler(Client client, CorbaControlHandlerServant cchServant, Queue queue) {
        // set the Thread name
        setName("clientinterface.CorbaDataHandler");
        
        _client = client;
        _cchServant = cchServant;
        _queue = queue;
        _logger.write(toString(), Logger.SYSINIT, "created");
    }
    
//---PUBLIC METHODS---
    
    /**
     * This method loops round until such a point as we shutdown. It keeps
     * grabbing data items from it's Queue, then sends them on to the connected
     * client over CORBA.
     */
    public void run() {
        // get a queue
        _queueID = _queue.getQueue();
        // we'll keep going until someone implements a "shutdown" :)
        run = true;
        // loop sending data
        while(run) {
            try {
                String xml = (String) _queue.get(_queueID);
                // if it's not null (which could happen if we're "released")
                // send it on to the client that we're connected to.
                if(xml != null) {
                    try {
                        _client.receiveXML(xml);
                    }
                    catch (org.omg.CORBA.COMM_FAILURE e) {
                        // lets stop sending, the client has quit
                        run = false;
                        _logger.write(toString(), Logger.ERROR, "Connection failure (COMM_FAILURE), client shutdown? : "+e);
                    }
                    catch (org.omg.CORBA.TRANSIENT e) {
                        // lets stop sending, the client has quit
                        run = false;
                        _logger.write(toString(), Logger.ERROR, "Connection failure (TRANSIENT), client shutdown? : "+e);
                    }
                }
            }
            catch(InvalidQueueException e) {
                // lets stop sending.
                run = false;
                _logger.write(toString(), Logger.ERROR, "Queue failure: "+e);
            }
        }
        // disconnect the servant above us, or at least try
        _cchServant.disconnect();
        // if we get here we've been told to stop
        _logger.write(toString(), Logger.SYSMSG, "Shutting Down");
        // remove ourselves from the queue
        _queue.removeQueue(_queueID);
    }
    
    /**
     * Method to shutdown this Data Handler. All this actually does
     * is set a flag which the main loop will see and commence shutting
     * down. This method will return before the main loop stops.
     */
    public void shutdown() {
        // set the main loop to stop
        run=false;
        // if the main loop is waiting for data it won't notice the
        // above flag to stop. This bumps it out of the blocked get.
        _queue.releaseQueue(_queueID);
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

    /**
     * Overridden for debugging purposes
     * to see when an instance of this class
     * is destroyed
     */
    protected void finalize() throws Throwable {
        _logger.write(this.toString(), Logger.DEBUG, "finalized by GC");
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = RootFilter.NAME;
    
    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * The Queue we'll use for buffering data to the client.
     */
    private Queue _queue;
    
    /**
     * The "servant" part of the client we're connected to.
     */
    private Client _client;
    
    /**
     * Our queue number within our Queue
     */
    private int _queueID;
    
    /**
     * The flag that dictates whether the main loop should *completely* exit
     */
    private boolean run;
    
    /**
     * A reference to our controlling class
     */
    private CorbaControlHandlerServant _cchServant;
    
//---STATIC ATTRIBUTES---

}
